MON    ; #######################################################
 ; POWER-UP PROGRAM FOR USE INDEPENDENTLY (WITHOUT BASIC
 ; ROMS IN PLACE) OR IN CONJUNCTION WITH EXIT AND RE-ENTRY
 ; TO BASIC UP RESET TO 0000 (BEFORE DISC SYSTEM REBOOT).
 ; #######################################################
 ;
 KPLACE	EQU	401AH		; RAM STORAGE FOR STROKE
 	ORG	3000H		; MONITOR ENTRY POINT
 ;
 ; #######################################################
 ; OPTIONAL SCREEN-CLEAR.  IF THE EXIT CONDITION OF THE
 ; SCREEN IS IMPORTANT, THEN ELIMINATE CLEARING PROCESS
 ; #######################################################
 ;
 	DI			; KILL THE INTERRUPTIONS
 	LD	(42E2H),SP	; SAVE STACK POINTER
 	LD	SP,42E0H	; GET NEW STACK POINTER
 	PUSH	AF		; SAVE REGISTERS FOR 
 	PUSH	BC		; 	LATER DISPLAY
 	PUSH	DE		;	DURING THE
 	PUSH	HL		;	MONITOR ENTRY
 CLS	LD	HL,3C00H	; BEGINNING OF SCREEN
 	LD	DE,3C01H	; NEXT POSITION ON SCREEN
 	LD	BC,3FFH		; NUMBER OF SCREEN POS'NS
 	LD	(HL),20H	; PRINT SPACE 1ST POS'N
 	LDIR			; DO IT FOR WHOLE SCREEN
 ;
 ; #######################################################
 ; DISPLAY DISK/MONITOR/WARM START/USER ROUTINE MESSAGE
 ; #######################################################
 ;
 	LD	HL,MSG01	; GET FIRST MESSAGE
 	LD	DE,3CC0H	; GET DISPLAY POSITION
 	CALL	DISPLY		; AND DISPLAY ON SCREEN
 MENU	CALL	INPUT		; GET INPUT FROM KEYBOARD
 	CP	63H		; IS IT A LETTER "C"?
 	JR	Z,COLD		; GO TO COLDSTART ROUTINE
 	CP	64H		; IS IT A LETTER "D"?
  	JP	Z,0000		; GO TO DISK REBOOT
 	CP	6DH		; IS IT A LETTER "M"?
 	JR	Z,MONTOR	; GO TO MACHINE MONITOR
 	CP	75H		; IS IT A LETTER "U"?
 	JR	Z,USER		; GO TO USER ROUTINE
 	CP	77H		; IS IT A LETTER "W"?
 	JR	Z,WARM		; GO TO BASIC WARMSTART
 	JR	MENU		; LOOP BACK IF NONE
 ;
 ; #######################################################
 ; COLDSTART ROUTINE DUPLICATES LEVEL II ACTION TO 0075H.
 ; STACK POINTER NOT RESET HERE BECAUSE COLDSTART DOES IT.
 ; #######################################################
 ;
 COLD	DI			; INTERRUPTS TURNED OFF
 	XOR	A		; CLEAR ACCUMULATOR
 CLEAR	OUT 	(0FFH),A	; BEGIN WRITE-PORT LOOP
 	LD	HL,06D2H	; GET COMMUNICATION BLOCK
 	LD	DE,4000H	; GET COMMUNICATION AREA
 	LD	BC,0036H	; GET SIZE OF COMM. BLOCK
 	LDIR			; BLOCK MOVE COMM. AREA
  	DEC	A		; A = A-1 (A=0 AT START)
 	DEC	A		; A = A-1 (FE, FC, FD...)
 	JR	NZ,CLEAR	; CLEAR ALL EVEN PORTS
 	LD	B,27H		; GET 39 VAR. LOCATIONS
 LOOP1	LD	(DE),A		; BEGIN SETTING UP CHARS
 	INC	DE		; GET NEXT MEM POS'N
 	DJNZ	LOOP1		; LOOP THROUGH THEM ALL
 	JP	0075H		; REST OF BASIC STARTUP
 ;
 ; #######################################################
 ; USER ENTRY ROUTINE REQUIRES THAT A TWO-BYTE ADDRESS B
 ; STORED IN MEMORY AT 408E/408F (16526/16527), <=USR(0)>
 ; #######################################################
 ;
 USER	POP	HL		; BEGIN TO CLEAR THE
 	POP	DE		;	STACK OF ALL
 	POP	BC		;	REGISTERS PUSHED
 	POP	AF		;	AT ENTRY POINT
 	LD	SP,(42E2H)	; GET ORIGINAL STACK PTR
 	LD	HL,(408EH)	; GET USR(0) ENTRY POINT
 	JP	(HL)		; AND JUMP TO IT
 ;
 ; #######################################################
 ; WARM START ROUTINE RESTORES STACK BEFORE GOING TO BASIC
 ; #######################################################
 ;
 WARM	POP	HL		; RESTORE ALL REGISTERS
 	POP	DE		;	PUSHED ON STACK
 	POP	BC		;	WHEN MONITOR
 	POP	AF		;	WAS ENTERED
 	LD	SP,(42E2H)	; GET ORIGINAL STACK PTR
 	JP	06CCH		; GO TO BASIC WARMSTART
 ;
 ; #######################################################
 ; MACHINE LANGUAGE MONITOR IS SIMPLE, DISPLAYING REGISTER
 ; CONDITIONS AT ENTRY, AND ACCEPTING MEMORY CHANGES.  ALL
 ; REGISTERS ARE RESTORED UPON EXIT FROM THIS ROUTINE.
 ; #######################################################
 ;
 MONTOR	LD	HL,MSG02	; GET REGISTER DISPLAY
 	LD	DE,3CC0H	; GET SCREEN POSITION
 	CALL	DISPLY		; DISPLAY THE MESSAGE
 	LD	DE,3D00H	; GET SCREEN POSITION
 	LD	BC,(42E2H)	; GET STACK POINTER POS'N
 	PUSH	BC		; GET VALUE READY TO XFER
 	POP	HL		; TRANSFER BYTES TO HL
 	CALL	WORDER		; CONVERT & DISPLAY BYTE
 	PUSH	IX		; GET IT READY TO USE
 	POP	HL		; AND DO IT WITH IT
 	CALL	WORDER		; CONVERT & DISPLAY BYTES
 	PUSH	IY		; GET IT READY TO USE
 	POP	HL		; AND DO IT WITH IT
 	CALL 	WORDER		; CONVERT & DISPLAY IT
 	LD	HL,(42DEH)	; POINT TO AF STACK VALUE
 	CALL	WORDER		; CONVERT & DISPLAY BYTE
 	LD	HL,(42DCH)	; POINT TO BC STACK VALUE
 	CALL	WORDER		; CONVERT & DISPLAY BYTE
 	LD	HL,(42DAH)	; POINT TO DE STACK VALUE
 	CALL	WORDER		; CONVERT & DISPLAY BYTE
 	LD	HL,(42D8H)	; POINT TO HL STACK VALUE
 	CALL	WORDER		; CONVERT & DISPLAY BYTE
 	LD	A,R		; GET R REGISTER INTO A
 	LD	H,A		; GIVE IT FOR CONVERSION
 	LD	A,I		; GET I REGISTER INTO A
 	LD	L,A		; GIVE IT FOR CONVERSION
 	CALL	WORDER		; CONVERT & DISPLAY BYTE
 	LD	DE,3D80H	; GET NEW VIDEO POSITION
 	EX	AF,AF'		; TRANSFER OTHER VALUE
 	PUSH	AF		; READY TO XFER TO HL
 	POP	HL		; EFFECT TRANSFER TO HL
 	EX	AF,AF'		; RETURN ORIGINAL VALUE
 	CALL	WORDER		; CONVERT & DISPLAY BYTE
 	EXX			; TRANSFER BC, DE, HL
 	PUSH	HL		; FIRST SLIP BC ON STACK,
 	PUSH	DE		; THEN PUSH DE THERE,
 	PUSH	BC		; AND THEN THE LAST ONE.
 	EXX			; TRANSFER REGISTERS BACK
 	POP	HL		; GET VALUE ALL READY
 	CALL 	WORDER		; CONVERT & DISPLAY BYTE
 	POP	HL		; GET DE VALUE READY
 	CALL	WORDER		; CONVERT & DISPLAY BYTE
 	POP	HL		; GET HL VALUE READY
 	CALL	WORDER		; CONVERT & DISPLAY BYTE
 CHECK	LD	DE,3F00H	; GET NEW SCREEN POS'N
 	CALL	WORDER		; CONVERT & DISPLAY BYTE
 	INC	E		; BUMP SCREEN POSITION
 	LD	A,(HL)		; GET VALUE FROM MEMORY
 	CALL	HEXASC		; AND DISPLAY IT IN ASCII
 ;
 ; #######################################################
 ; ADDRESS MOD ROUTINE GETS KEYBOARD VALUE & LOOPS BACK
 ; #######################################################
 ;
 ADDMOD	CALL	INPUT		; GET VALUE FROM KEYBOARD
 	CP	'!'		; TEST FIRST IF EXECUTE
 	JR	Z,EXEC		; OUT IF EXECUTE COMMAND
 	CP	'/'		; TEST SECOND IF DATA
 	JP	Z,DATMOD	; OUT IF DATA MOD COMMAND
 	CP	'*'		; TEST THIRD IF RET MENU
 	JP	Z,CLS		; OUT TO MENU IF A STAR
 	CP	'0'		; SEE IF <0 CHARACTER
 	JR	C,ADDMOD	; LOOP BACK IF <0 CHAR.
 	CP	67H		; SEE IF >F CHARACTER
 	JR	NC,ADDMOD	; LOOP BACK IF >F CHAR.
 	CP	':'		; SEE IF <9 CHARACTER
 	JR	C,NUMBER	; GO TO NUMBER ROUTINE
 	CP	61H		; SEE IF >A CHARACTER
 	JR	C,ADDMOD	; LOOP BACK IF <A & >9
 NUMBER	LD	HL,3F04H	; GET ADDRESS SCREEN POSN
 	LD	DE,3F03H	; GET NEXT SCREEN POS'N
 	LD	BC,3		; GET THREE TOTAL POS'NS
 	LDIR			; AND MOVE THEM OVER
 	DEC	HL		; REPOSITION TO LAST CHAR
 	CP	60H		; COMPARE TO L.C. ALPHA
 	JR	C,ZIPBY		; IF NUMBERIC, THEN SKIP
 	SUB	20H		; ELSE CONVERT TO U.CASE
 ZIPBY	LD	(HL),A		; DISPLAY NEW CHARACTER
 NUM2	CALL	ASCHEX		; CONVERT DISPLAY TO HL
 	JR	CHECK		; AND LOOP BACK FOR MORE
 EXEC	LD	HL,3F06H	; POINT HL TO ADDRESS
 	CALL	ASCHEX		; AND CONVERT TO HEX
 	JP	(HL)		; HL CONTAINS ADDRESS
 ;
 ; #######################################################
 ; SUBROUTINE RESPONSIBLE FOR CONVERTING & DISPLAYING TWO-
 ; BYTE WORDS DELIVERED TO IT IN THE HL REGISTER.
 ; #######################################################
 ;
 WORDER	INC	E		; BUMP DE REGISTER ALONG
 	INC	E		; SOME MORE BUMPING ...
 	INC	E		; ... AND SOME MORE OF IT
 	LD	A,H		; GET FIRST BYTE OF IT
 	CALL	HEXASC		; CONVERT & DISPLAY BYTE
 	LD	A,L		; GET SECOND BYTE OF IT
 	CALL	HEXASC		; CONVERT & DISPLAY BYTE
 	INC	E		; AND POSITION SCREEN
 	RET			; AND BACK TO CALLER 
 ;
 ; #######################################################
 ; HEXADECIMAL TO ASCII CONVERSION FOR TWO-BYTE WORDS
 ; #######################################################
 ;
 HEXASC	PUSH	AF		; SAVE BYTE IN AF REG.
 	AND	0F0H		; AND MASK OUT LOW NYBBLE
 	RRCA			; BEGIN ROTATING BIT ...
 	RRCA			; ... IN ORDER TO TEST...
 	RRCA			; ... THE LOW NYBBLE ...
 	RRCA			; ... ALL BY ITSELF.
 	CALL	HXAC		; BYTE RANGE EVALUATION
 	LD	(DE),A		; DISPLAY ON THE SCREEN
 	INC	E		; AND MOVE TO NEXT POS'N
 	POP	AF		; RESTORE ORIGINAL BYTE
 	AND	0FH		; AND MASK OUT HIGH BITS
 	CALL	HXAC		; RANGE EVALUATION ROUT.
 	LD	(DE),A		; AND DISPLAY ON SCREEN
 	INC	E		; MOVE TO NEXT VIDEO POSN
 	RET			; AND BACK TO CALLER
 ;
 ; #######################################################
 ; HEX-ASCII RANGE EVALUATION FOR 0 TO 9 AND A TO F AREAS
 ; #######################################################
 ;
 ;
 HXAC	CP	0AH		; CHECK AGAINST 10 DEC.
 	JR	NC,NEXTX	; IF >10 THEN ALPHA CHAR
 	ADD	A,30H		; ELSE NUMERIC - CONVERT
 	RET			; AND BACK TO CALLER
 NEXTX	ADD	A,37H		; ALPHABETIC - CONVERT
 	RET			; AND BACK TO CALLER
 ;
 ; #######################################################
 ; ASCII TO HEXADECIMAL CONVERSION FOR TWO-BYTE WORDS
 ; #######################################################
 ;
 ;
 ASCHEX	CALL	ACHX		; RANGE EVALUATION ROUT.
 	LD	C,A		; SAVE PART OF CONVERSION
 	DEC	HL		; AND GET NEXT BYTE
 	CALL	LLLLS		; EXECUTE LEFT ROTATES
 	ADD	A,C		; AND ADD TO MAKE BYTE
 	LD	C,A		; SAVE IT BACK IN C REG.
 	DEC	HL		; AND MOVE TO NEXT BYTE
 AX2	CALL	ACHX		; DO RANGE EVALUATION
 	LD	B,A		; AND SAVE IT IN B REG.
 	DEC	HL		; GET FINAL BYTE READY
 	CALL	LLLLS		; AND EXECUTE LEFT ROTATE
 	ADD	A,B		; CREATE COMPLETE BYTE
 	LD	B,A		; AND SAVE IT IN B REG.
 	PUSH	BC		; GET 2 BYTES TO XFER
 	POP	HL		; TRANSFER TO HL REGISTER
 	RET			; AND BACK TO CALLER
 LLLLS	CALL	ACHX		; EVALUATE RANGE OF CHAR
 	RLCA			; BEGIN LEFT ROTATES ...
 	RLCA			; ... WHICH WILL POS'N...
 	RLCA			; ... BYTE READY FOR ...
 	RLCA			; ... CONVERSION.
 	RET			; AND GO BACK TO CALLER
 ACHX	LD	A,(HL)		; GET BYTE FROM SCREEN
 	CP	40H		; CHECK AGAINST ALPHA
 	JR	NC,NEXTZ	; IF NUMERIC THEN JUMP
 	SUB	30H		; ELSE NUMBER TO HEX
 	RET			; AND BACK TO CALLER
 NEXTZ	SUB	37H		; THEN IT'S ALPHA TO HEX
 	RET			; AND BACK TO CALLER
 ;
 ; #######################################################
 ; ROUTINE TO ACCEPT DATA FOR INPUT TO DISPLAYED MEMORY
 ; LOCATIONS, WITH EXIT TO MENU UPDATE OR MEMORY INCREMENT
 ; #######################################################
 ;
 DATMOD	CALL	INPUT		; GET CHAR. FROM KEYBRD.
 	CP	'.'		; IS IT A PERIOD?
 	JP	Z,ADEXIT	; IF SO, EXIT TO ADDR MOD
 	CP	0DH		; IS IT A CARRIAGE RET?
 	JR	Z,NEXTD		; IF SO, GO TO NEXT DATA
 	CP	'0'		; BEGIN TESTING FOR RANGE
 	JR	C,DATMOD	; BACK IF <0 CHARACTER
 	CP	67H		; CHECK AGAINST LC ALPHA
 	JR	NC,DATMOD	; IF >F THEN GO BACK
 	CP	':'		; TEST FOR NUMERIC >9
 	JR	C,INDATA	; IF <9 THEN GET DATA
 	CP	61H		; TEST FOR ALPHA <A
 	JR	C,DATMOD	; IF <A THEN GO BACK
 INDATA	LD	HL,3F0AH	; READY THE SCREEN PTR.
 	PUSH	AF		; SAVE VALUE FROM KEYBRD
 	LD	A,(HL)		; GET LOWER ASCII NYBBLE
 	LD	(3F09H),A	; AND PUT IN HIGH POS'N
 	POP	AF		; RESTORE VALUE FROM KBD
 	CP	60H		; CHECK AGAINST LC ALPHA
 	JR	C,ZIPZY		; IF LC, THEN SKIP PAST
 	SUB	20H		; ELSE CONVERT TO U.C.
 ZIPZY	LD	(HL),A		; AND DISPLAY ON SCREEN
 	CALL	AX2		; SINGLE BYTE CONVERSION
 	LD	A,H		; SAVE ONE BYTE IN A REG.
 	PUSH	AF		; SAVE THAT NOW ON STACK
 	LD	HL,3F06H	; POINT HL TO SCREEN POSN
 	CALL	ASCHEX		; EVALUATE ADDRESS THERE
 	POP	AF		; RESTORE VALUE TO SHOW
 	LD	(HL),A		; AND PUT ON THE SCREEN
 	JR	EXIT1		; NOW GO OUT AND STORE
 NEXTD	LD	HL,3F06H	; POINT HL TO SCREEN POSN
 	CALL	ASCHEX		; CONVERT ADDRESS TO HEX
 	INC	HL		; MOVE OVER TO NEXT POSN
 EXIT1	LD	DE,3F00H	; POINT DE ADDR. DISPLAY
 	CALL	WORDER		; AND CONVERT/DISPLAY IT
 	INC	E		; GO TO NEXT SCREEN POSN
 	LD	A,(HL)		; GET VALUE FROM MEMORY
 	CALL	HEXASC		; AND CONVERT FOR DISPLAY
 	JR	DATMOD		; AND BACK FOR SOME MORE
 ADEXIT	LD	HL,3F06H	; POINT TO PRESENT ADDR.
 	JP	NUM2		; AND BACK TO UPDATE MENU
 ;
 ; #######################################################
 ; COMPLETE KEYBOARD ROUTINE BELOW MAY BE CALLED BY NOT
 ; ONLY THE MONITOR ROUTINES, BUT ANY ROUTINES NEEDING AN
 ; UPPER/LOWER CASE, AUTOREPEAT, BEEP DRIVER.  FOR FURTHER
 ; DESCRIPTION, SEE SUPPLEMENT TO CHAPTER ON KEYBOARD I/O
 ; #######################################################
 ;
 INPUT	LD	HL,4036H	; SAME AS LII BUFFER
 	LD	BC,3801H	; FIRST KEYBOARD ROW
 	LD	D,0		; COUNTER FOR COLUMNS
 KEYPRS	LD	A,(BC)		; RETRIEVE ROW CONTENTS
 	LD	E,A		; SAVE IT TEMPORARILY
 	AND	E		; SET FLAGS FOR TESTING
 	JR	NZ,STROKE	; NOT ZERO IF KEY PRESSED
 	LD	(HL),A		; SAVE CURRENT VALUE
 RECHEK	INC	D		; INCREMENT ROW COUNTER
 	INC	L		; INCREMENT STORAGE ADDR.
 	RLC	C		; GET NEXT KEYBOARD COL.
 	LD	A,C		; GET VALUE INTO ACCUM.
 	SUB	80H		; LAST ROW IS 3880 HEX
 	JR	NZ,KEYPRS	; NEXT CHECK IF NOT DONE
 	LD	B,7		; COUNTER OF KEYBRD ROWS
 CLRMEM	DEC	L		; START COUNTING BACKWARD
 	ADD	A,(HL)		; AND ADD IT UP IN ACCUM.
 	DJNZ	CLRMEM		; AND DO IT FOR 7 ROWS
 	AND	A		; TEST FOR ANY KEY DOWN
 	LD	A,0		; A=0, FLAGS ARE INTACT
 	RET	NZ		; BACK IF KEYS IN USE
 	LD	(KPLACE),A	; ELSE DELAY GETS RESET
 	RET			; AND GO BACK ANYWAY
 STROKE	AND	(HL)		; CHECK KEYSTROKE STORAGE
 	JR	Z,FOUND		; NEW KEY IF NOT SAME ONE
 	LD	A,(KPLACE)	; NEW CHECK SPECIAL STORE
 	INC	A		; LET STORE = STORE + 1
 	LD	(KPLACE),A	; AND PUT IN BACK THERE
 	CP	0FFH		; CHECK IF IT IS AT END
 	JR	Z,DECA		; IF SO, THEN HOLD THERE
 	PUSH	BC		; SAVE ROW COUNTER REG.
 	LD	B,0FFH		; GET DELAY VALUE INTO B
 TMWSTE	DJNZ	TMWSTE		; AND DELAY JUST A BIT
 	POP	BC		; RESTORE ROW COUNTER
 	JR	RECHEK		; AND BACK TO CHECK NEXT
 DECA	DEC	A		; LET A = A - 1 (STORE)
 	LD	(KPLACE),A	; AND PUT IT IN STORAGE
 	LD	A,E		; GET KEYBOARD BYTE BACK
 FOUND	LD	(HL),E		; STORE IT IS STROKE AREA
 	LD	A,D		; GET ROW COUNTER FROM D
 	RLCA			; AND BEGIN PROCESS OF...
 	RLCA			; ...CONVERTING IT...
 	RLCA			; ...TO AN OFFSET VALUE
 	LD	D,A		; AND PUT IT BACK IN D
 	LD	C,1		; GET NUMBER ONE READY
 BACKUP	LD	A,C		; ACCUM. HAS C FOR MASK
 	AND	E		; TEST IF C = KEYSTROKE
 	JR	NZ,AROUND	; IF NOT, THEN GO AROUND
 	INC	D		; ELSE D = ROW + COLUMN
 	RLC	C		; C SET TO NEXT COLUMN
 	JR	BACKUP		; GO BACK AND TEST AGAIN
 AROUND	LD	A,(3880H)	; GET SHIFT ROW FOR TEST
 	LD	B,A		; AND SAVE IT IN B REG.
 	LD	A,D		; GET ROW COUNTER BACK
 	ADD	A,40H		; AND CONVERT TO ASCII
 	CP	60H		; IS IT UP/LW/GRAFIX/CTRL
 	JR	NC,JUMP1	; GO OUT IF GRAPHICS
 	LD	D,A		; SAVE PARTLY CONVERTED
 	LD	A,(3840H)	; GET VALUE FOUND 7TH ROW
 	AND	10H		; CHECK IF DOWN ARROW
 	JR	NZ,CNTROL	; IF SO, PRODUCE CONTROL
 	LD	A,D		; ELSE GET VALUE BACK
 	RRC	B		; B BUMPS INTO CARRY FLAG
 	JR	C,GOAWAY	; IF CARRY, THEN SHIFT
 	ADD	A,20H		; IF NOT THEN LOWER CASE
 	JR	GOAWAY		; AND GET OUT OF ROUTINE
 CNTROL	LD	A,D		; IF CONTROL CODE, GET IT
 	SUB	40H		; GET RID OF ASCII MASK
 	JR	GOAWAY		; AND GET OUT OF ROUTINE
 JUMP1	SUB	70H		; THE BALANCE OF THE
 	JR	NC,JUMP2	;    ROUTINE BELOW UP TO
 	ADD	A,40H		;    THE BEEP SECTION IS
 	CP	3CH		;    VIRTUALLY IDENTICAL
 	JR	C,JUMP3		;    TO THE KEYBOARD
 	XOR	10H		;    DETERMINATION SUB-
 JUMP3	RRC	B		;    ROUTINE FOUND IN
 	JR	NC,GOAWAY	;    ROM.  THIS ENTIRE
 	XOR	10H		;    ROUTINE IS REPEATED
 	JR	GOAWAY		;    BECAUSE IT CAN
 JUMP2	RLCA			;    SERVE AS AN EXACT
 	RRC	B		;    REPLACEMENT FOR
 	JR	NC,JUMP4	;    THE LEVEL II BASIC
 JUMP4	LD	HL,TABLET	;    ROUTINES WHEN (IF)
 	LD	C,A		;    THE ROMS HAVE BEEN
 	LD	B,0		;    REMOVED, REPLACED,
 	ADD	HL,BC		;    DISABLED, ETC.
 	LD	A,(HL)		;    SEE KEYBOARD ROUTINE
 	JR	GOAWAY		;    RUNDOWN ELSEWHERE
 	INC	A		;    IN THIS BOOK......
 TABLET	DEFW	0D0DH		; CARR. RET. / CARR. RET.
 	DEFW	1F1FH		; CLEAR SCRN / CLEAR SCRN
 	DEFW	0101H		; BREAK KEY / BREAK KEY
 	DEFW	1B5BH		; EDIT ESCAPE / UP ARROW
 	DEFW	000AH		; NO CHANGE / LINEFEED
 	DEFW	1808H		; BACKSP. LINE / BACKSP.
 	DEFW	1909H		; 32-CHAR MODE / HOR TAB
 	DEFW	2020H		; SPACE / SPACE
 GOAWAY	LD	D,A		; SAVE VALUE IN D REG.
 	LD	BC,180H		; DEBOUNCE VALUE AND
 DELAYS	DEC	BC		;    IMPLEMENTED AT THIS
 	LD	A,B		;    POINT FOR ACTUAL
 	OR	C		;    KEYBOARD SCAN WHEN
 	JR	NZ,DELAYS	;    CHARACTERS ARE
 	LD	A,D		; GET STORED VALUE BACK
 	PUSH	AF		; SAVE ACCUM. & FLAGS
 	LD	B,40H		; GET BEEP LENGTH VALUE
 	LD	A,(403DH)	; GET STATUS OF SCREEN
 	AND	0FDH		; MASK SCREEN CHANGE
 	LD	H,A		; STORE MSB IN H REG.
 	OR	2		; SET BIT 1 TO BE ON
 	LD	L,A		; STORE ALT. MSB IN L REG
 BEEPER	LD	A,L		; GET ALT. MSB TO OUTPUT
 	OUT	(0FFH),A	; AND OUTPUT RISING WAVE
 	LD	A,H		; GET NORMAL MSB CHAR.
 	OUT 	(0FFH),A	; AND OUTPUT FALLING WAVE
 	PUSH	BC		; SAVE NOTE LENGTH VALUE
 	LD	B,40H		; GET FREQUENCY DELAY
 FREQCY	DJNZ	FREQCY		; AND WAIT A LITTLE WHILE
 	POP	BC		; NOW RESTORE LENGTH VAL.
 	DJNZ	BEEPER		; AND GO BACK THAT LENGTH
 	POP	AF		; RESTORE ORIGINAL CHAR.
 	RET			; AND BACK TO CALLER
 ;
 ; #######################################################
 ; ROUTINE BELOW TAKES SIMPLE MESSAGES AND DISPLAYS THEM
 ; #######################################################
 ;
 DISPLY	LD	A,(HL)		; GET FIRST MSG CHARACTER
 	AND	A		; TEST IF 0 OR CHARACTER
 	RET	Z		; BACK TO CALLING ROUTINE
 	LD	(DE),A		; DISPLAY THE CHARACTER
 	INC	HL		; GET NEXT MSG CHARACTER
 	INC	DE		; GET NEXT DISPLAY POS'N
 	JR	DISPLY		; AND TEST NEXT CHARACTER
 ;
 ; #######################################################
 ; MESSAGES FOR DISPLAY FOLLOW.  IN A STRIPPED SOFTWARE
 ; MONITOR CONFIGURATION, THESE MESSAGES MAY BE ELIMINATED
 ; AND DISPLAY PARAMETERS CHANGED ELSEWHERE IN THE LISTING
 ; #######################################################
 ;
 MSG01	DEFM	'<C>OLDSTART, <D>ISKBOOT, <M>ONITOR, '
 	DEFM	'<U>SERJUMP, <W>ARMSTART? '
 	DEFB	8FH
 	DEFB	00
 MSG02	DEFM	'Z80 REGISTERS AT MONITOR ENTRY: '
 	DEFM	'                                '
 	DEFM	'SP=     IX=     IY=     AF=     '
 	DEFM	'BC=     DE=     HL=     RI=     '
 	DEFM	'ALTERNATE REGISTERS AT MONITOR E'
 	DEFM	'NTRY (NOT USED IN LEV II BASIC):'
 	DEFM	'AF=     BC=     DE=     HL=     '
 	DEFM	'                                '
 	DEFM	'USE PERIOD (.) TO ENTER AN ADDRE'
 	DEFM	'SS, SLASH (/) TO ENTER DATA.  TO'
 	DEFM	'EXIT TO MONITOR MENU, TYPE STAR '
 	DEFM	'(*), TO EXECUTE AT ADDRESS SHOWN'
 	DEFM	'ON SCREEN, TYPE EXCLAMATION POIN'
 	DEFM	'T (!)                           '
 	DEFB	00
 XXXXXX	EQU	$
 ;
 	END
